# Generated by Django 1.10.2 on 2016-12-12 03:14

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0008_alter_user_username_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="AAXenforo2User",
            fields=[
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        related_name="aaxenforo2",
                        serialize=False,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                ("username", models.CharField(max_length=254)),
                ("remote_user", models.BigIntegerField(default=0)),
            ],
        ),
    ]
